==> thx_core-0.2.10.tar.gz: thx_core-0.2.10/thx/core/management/commands/makematerializedviewmigrations.py <==
# -*- coding: utf-8 -*-
from typing import List

from django.apps import apps
from django.core.management.base import no_translations
from django.core.management.commands.makemigrations import Command as MakeMigrationsCommand
from django.db import migrations
from django.db.migrations.autodetector import MigrationAutodetector
from django.db.migrations.loader import MigrationLoader
from django.db.migrations.migration import Migration
from django.db.migrations.state import ProjectState

from thx.core.models import MaterializedDbViewModel


class Command(MakeMigrationsCommand):
    help = "Creates new migration(s) also for the materialized views."

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            '-m', '--materialized-views', type=str, required=False,
            help="Comma separated materialized views."
                 " Indicate using format `app_label.MyMaterializedViewModel`",
        )

    @no_translations
    def handle(self, *app_labels, **options) -> None:
        materialized_views = options.pop('materialized_views', None)
        if materialized_views:
            materialized_views = materialized_views.replace(" ", "").split(",")
        super().handle(*app_labels, **options)
        changes = self.get_materialized_view_changes(app_labels, materialized_views)

        if changes:
            loader = MigrationLoader(None, ignore_no_migrations=True)
            autodetector = MigrationAutodetector(
                loader.project_state(),
                ProjectState.from_apps(apps),
            )
            changes = autodetector.arrange_for_graph(changes=changes, graph=loader.graph)
            self.write_migration_files(changes)
        else:
            self.stdout.write("No MaterializedViews detected")

    def get_materialized_view_changes(self, app_labels=None, materialized_views=None) -> dict:
        return {
            k: self.create_migrations(k, v)
            for k, v in self.find_materialized_views(app_labels, materialized_views).items()
        }

    @staticmethod
    def find_materialized_views(app_labels=None, materialized_views=None) -> dict:
        mvs = {}
        app_configs = [
            apps.app_configs.get(app_label) for app_label in app_labels
        ] if app_labels else apps.app_configs.values()

        selected_mv = None
        if materialized_views:
            selected_mv = {}
            for mv in materialized_views:
                app_label, model = mv.split(".")
                if app_label not in selected_mv:
                    selected_mv[app_label] = []
                selected_mv[app_label].append(model)
            
        for app_config in app_configs:
            for model in app_config.get_models(False, False):
                if issubclass(model, MaterializedDbViewModel):
                    if selected_mv is not None:
                        if model.__name__ not in selected_mv.get(app_config.label, []):
                            continue

                    if app_config.label not in mvs:
                        mvs[app_config.label] = []
                    mvs[app_config.label].append(model)
        return mvs

    @staticmethod
    def create_migrations(app_label, mv_models: List[MaterializedDbViewModel]):
        migration = Migration(name=f"tmp_{app_label}", app_label=app_label)
        operations = []
        for mv_model in mv_models:
            sql_delete = mv_model.get_sql_delete()
            sql_create = mv_model.get_sql_create()
            operations.extend([
                migrations.RunSQL(sql=sql_delete, reverse_sql=migrations.RunSQL.noop),
                migrations.RunSQL(sql=sql_create, reverse_sql=migrations.RunSQL.noop),
            ])
        migration.operations = operations
        return [migration]

==> thx_core-0.2.14.tar.gz: thx_core-0.2.14/thx/core/management/commands/makematerializedviewmigrations.py <==
# -*- coding: utf-8 -*-
from typing import List

from django.apps import apps
from django.core.management.base import no_translations
from django.core.management.commands.makemigrations import Command as MakeMigrationsCommand
from django.db import migrations
from django.db.migrations.autodetector import MigrationAutodetector
from django.db.migrations.loader import MigrationLoader
from django.db.migrations.migration import Migration
from django.db.migrations.state import ProjectState

from thx.core.management.commands import ThxBaseCommand
from thx.core.models import MaterializedDbViewModel


class Command(ThxBaseCommand, MakeMigrationsCommand):
    help = "Creates new migration(s) also for the materialized views."

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            '-m', '--materialized-views', type=str, required=False,
            help="Comma separated materialized views."
                 " Indicate using format `app_label.MyMaterializedViewModel`",
        )

    @no_translations
    def handle(self, *app_labels, **options) -> None:
        materialized_views = options.pop('materialized_views', None)
        if materialized_views:
            materialized_views = materialized_views.replace(" ", "").split(",")
        super().handle(*app_labels, **options)
        changes = self.get_materialized_view_changes(app_labels, materialized_views)

        if changes:
            loader = MigrationLoader(None, ignore_no_migrations=True)
            autodetector = MigrationAutodetector(
                loader.project_state(),
                ProjectState.from_apps(apps),
            )
            changes = autodetector.arrange_for_graph(changes=changes, graph=loader.graph)
            self.write_migration_files(changes)
        else:
            self.stdout.write("No MaterializedViews detected")

    def get_materialized_view_changes(self, app_labels=None, materialized_views=None) -> dict:
        return {
            k: self.create_migrations(k, v)
            for k, v in self.find_materialized_views(app_labels, materialized_views).items()
        }

    @staticmethod
    def find_materialized_views(app_labels=None, materialized_views=None) -> dict:
        mvs = {}
        app_configs = [
            apps.app_configs.get(app_label) for app_label in app_labels
        ] if app_labels else apps.app_configs.values()

        selected_mv = None
        if materialized_views:
            selected_mv = {}
            for mv in materialized_views:
                app_label, model = mv.split(".")
                if app_label not in selected_mv:
                    selected_mv[app_label] = []
                selected_mv[app_label].append(model)

        for app_config in app_configs:
            for model in app_config.get_models(False, False):
                if issubclass(model, MaterializedDbViewModel):
                    if selected_mv is not None:
                        if model.__name__ not in selected_mv.get(app_config.label, []):
                            continue

                    if app_config.label not in mvs:
                        mvs[app_config.label] = []
                    mvs[app_config.label].append(model)
        return mvs

    @staticmethod
    def create_migrations(app_label, mv_models: List[MaterializedDbViewModel]):
        migration = Migration(name=f"tmp_{app_label}", app_label=app_label)
        operations = []
        for mv_model in mv_models:
            sql_delete = mv_model.get_sql_delete()
            sql_create = mv_model.get_sql_create()
            operations.extend([
                migrations.RunSQL(sql=sql_delete, reverse_sql=migrations.RunSQL.noop),
                migrations.RunSQL(sql=sql_create, reverse_sql=migrations.RunSQL.noop),
            ])
        migration.operations = operations
        return [migration]
